service system/alsasound/cards {
	need = system/bootmisc;
	use = system/coldplug system/modules/depmod system/modules;
	script start = {
		# List of drivers for each card.
		for mod in `/sbin/modprobe -c | /usr/bin/awk '$1 == "alias" && $2 ~ /^snd-card-[[:digit:]]$/ { print $2 } {}'`
		do
			echo "Loading ${mod}"
			/sbin/modprobe ${mod}
		done

		# Fall back on the automated aliases,
		# if we do not have ALSA configured properly...
		[ -d /proc/asound -a -z "`/bin/grep ' no soundcards ' /proc/asound/cards`" ] && exit 0
		echo "Could not detect custom ALSA settings.  Loading all detected alsa drivers."
		for mod in `/sbin/modprobe -c | /usr/bin/awk '$2~ /^pci:/ && $3~ /^snd.*/ { print $3 }' | sort -u`
		do
			echo "Loading ${mod}"
			/sbin/modprobe ${mod}
		done
		if ! [ -d /proc/asound -a -z "`/bin/grep ' no soundcards ' /proc/asound/cards`" ]
		then
			echo "Unable to find any ALSA drivers. Have you compiled alsa-drivers correctly?"
			exit 1
		fi
	};
	script stop = {
		# this unload all modules with snd*
		for mod in `/sbin/lsmod | /bin/sed -ne '2,$ { s/ .*$//; /^snd$/ p }'`
		do
			echo "Unloading ${mod}"
			/sbin/modprobe -r ${mod}
		done
	};
}

service system/alsasound/oss {
	need = system/bootmisc;
	use = system/coldplug system/modules/depmod system/modules;
	script start = {
		for mod in `/sbin/modprobe -l | /bin/grep "snd.*oss" | /bin/sed -e "s:\/.*\/::" -e "s:\..*::"`
		do
			echo "Loading ${mod}"
			/sbin/modprobe ${mod}
		done
	};
	script stop = {
		for mod in `/sbin/lsmod | /bin/sed -ne '2,$ { s/ .*$//; /^snd.*oss$/ p }'`
		do
			echo "Unloading ${mod}"
			/sbin/modprobe -r ${mod}
		done
	};
}

service system/alsasound/seq {
	need = system/bootmisc;
	use = system/coldplug system/modules/depmod system/modules;
	script start = {
		# We want to ensure snd-seq is loaded as it is needed for things like
		# timidity even if we do not use a real sequencer.
		for mod in `/sbin/modprobe -l | /bin/sed -ne '{ s|.*/\([^/]*\)\.ko$|\1|; /^snd[_-]seq/ { /oss/ !p } }'`
		do
			echo "Loading ${mod}"
			/sbin/modprobe ${mod}
		done
		for mod in `/usr/bin/awk -F, '$2~ /^empty$/ { print $1 }' /proc/asound/seq/drivers`
		do
			echo "Loading ${mod}"
			/sbin/modprobe ${mod}
		done
	};
	script stop = {
		for mod in `/sbin/lsmod | /bin/sed -ne '2,$ { s/ .*$//; /^snd[_-]seq/ { /oss/ !p} }'`
		do
			echo "Unloading ${mod}"
			/sbin/modprobe -r ${mod}
		done
	};
}

service system/alsasound/ioctl32 {
	need = system/bootmisc;
	use = system/coldplug system/modules/depmod system/modules;
	script start = {
		# We want to ensure snd-ioctl32 is loaded as it is needed for 32bit
		# compatibility
		for mod in `/sbin/modprobe -l | /bin/sed -ne '{ s|.*/\([^/]*\)\.ko$|\1|; /^snd[_-]ioctl32/ p}'`
		do
			echo "Loading ${mod}"
			/sbin/modprobe ${mod}
		done
	};
	script stop = {
		for mod in `/sbin/lsmod | /bin/sed -ne '2,$ { s/ .*$//; /^snd[_-]ioctl32/ p }'`
		do
			echo "Unloading ${mod}"
			/sbin/modprobe -r ${mod}
		done
	};
}

service system/alsasound/mixerstate {
	need = system/bootmisc system/alsasound;
	env asoundcfg=/etc/asound.state;
	script start = {
		if [ ! -r "${asoundcfg}" ]
		then
			echo "No mixer config in ${asoundcfg}, you have to unmute your card!"
			# this is not fatal!
		elif [ -x /usr/sbin/alsactl ]
		then
			for CARDNUM in `/usr/bin/awk '/: / { print $1 }' /proc/asound/cards`
			do
				/usr/sbin/alsactl -f ${asoundcfg} restore ${CARDNUM} &
			done
			wait
		else
			echo "ERROR: Cannot find alsactl, did you forget to install media-sound/alsa-utils?"
			exit 1
		fi
		exit 0
	};
	exec stop = /usr/sbin/alsactl -f ${asoundcfg} store;
}

service system/alsasound {
	need = system/bootmisc;
	use = system/coldplug system/alsasound/cards system/alsasound/ioctl32 system/alsasound/seq system/alsasound/oss system/modules/depmod system/modules;
	also_stop = system/alsasound/cards system/alsasound/ioctl32 system/alsasound/seq system/alsasound/oss;
	env alsascrdir=/etc/alsa.d;
	script start = {
		for DRIVER in `/sbin/lsmod | /usr/bin/awk '$1~/^snd.*/{print $1}'`
		do
			TMP=${DRIVER##snd-}
			TMP=${TMP##snd_}
			if [ -x "${alsascrdir}/${TMP}" ]
			then
				echo "  Running: ${alsascrdir}/${TMP} ..."
				${alsascrdir}/${TMP}
			fi
		done

		if [ ! -d /proc/asound -o ! -z "`/bin/grep ' no soundcards ' /proc/asound/cards`" ]
		then
			echo "ERROR: Failed to load necessary drivers"
			exit 1
		fi
	};
	script stop = {
		# Kill processes holding open sound devices
		/bin/fuser -k /dev/admmidi? /dev/adsp? /dev/amidi? /dev/audio* /dev/dmfm* \
			/dev/dmmidi? /dev/dsp* /dev/dspW* /dev/midi0? /dev/mixer? /dev/music \
			/dev/patmgr? /dev/sequencer* /dev/sndstat /proc/asound/dev/* \
			/dev/sound/* /dev/snd/* >/dev/null 2>&1

		# remove all sequencer connections if any
		[ -f /proc/asound/seq/clients -a -x /usr/bin/aconnect ] && \
			/usr/bin/aconnect --removeall
		/sbin/rmmod soundcore >/dev/null 2>&1
		/sbin/rmmod gameport >/dev/null 2>&1
		exit 0
	};
}
