# This one should be ran first, and have no dependencies.
# Its purpose is to mount /proc /dev and /sys.
service system/initial/mountvirtfs {
	critical;
	script start = {
		# test, if all necasary directories exists.
		for dir in /proc /sys
		do
			if [ ! -d ${dir} ] # no?
			then
				echo "The dir \"${dir}\" is missing, cant mount it!" >&2
				echo "Please sulogin, remount rw and create them." >&2
				exit 1 # It can't work. Critical!
			fi
		done

		mount -n -t proc proc /proc &
		mount -n -t sysfs sys /sys &

		wait
		exit 0
	};
}

service system/initial/filldev {
	need = system/initial/mountvirtfs;
	use = system/udev/mountdev;
	critical; # This script can't be fail.
	script start = {
		echo "Mounting devpts at /dev/pts ..."
		/bin/mkdir -p /dev/pts && /bin/mount -n -t devpts -o gid=5,mode=0620 none /dev/pts &

		echo "Mounting devshm at /dev/shm ..."
		/bin/mkdir -p /dev/shm && /bin/mount -n -t tmpfs none /dev/shm &

		# who needs this? only user, who needs this.
		# so i think, we should make this link, if somebody needs this.
		[ ! -e /dev/MAKEDEV -a /sbin/MAKEDEV ] && \
				/bin/ln -fs /sbin/MAKEDEV /dev/MAKEDEV &

		wait
		exit 0
	};
}

service system/initial/loglevel {
	# Set the console loglevel to 1 for a cleaner boot
	# the logger should anyhow dump the ring-0 buffer at start to the
	# logs, and that with dmesg can be used to check for problems
	exec start = /bin/dmesg -n 1;
	# Increase the debug level a bit
	exec stop = /bin/dmesg -n 2;
}

# This is the basic system/initial
virtual system/initial {
	critical;
	need = system/initial/loglevel system/initial/mountvirtfs system/initial/filldev;
	use = system/selinux/dev;
	use = system/udev;
}
