service system/keymaps {
	need = system/bootmisc system/sysctl;
	env CONFDIR=/etc/console;
	env CONFFILEROOT=boottime;
	env EXT=kmap;
	env CONFFILE=${CONFDIR}/${CONFFILEROOT}.${EXT}.gz;

	script start = {
		# Don't fail on error
		CONSOLE_TYPE=`fgconsole 2>/dev/null` || CONSOLE_TYPE="unknown"
		# fail silently if loadkeys not present (yet).
		command -v loadkeys >/dev/null 2>&1 || exit 0

		unicode_start_stop() {
			# Switch unicode mode by checking the locale.
			# This will be needed before loading the keymap.
			[ -x /usr/bin/unicode_start ] || [ -x /usr/bin/unicode_start ] || return
			[ -x /usr/bin/unicode_stop ] || [ -x /usr/bin/unicode_stop ] || return
			[ -r /etc/environment ] || return

			for var in LANG LC_ALL LC_CTYPE
			do
				value=`/bin/egrep "^[^#]*${var}=" /etc/environment | /usr/bin/tail -n1 | /usr/bin/cut -d= -f2`
				eval ${var}=${value}
			done

			CHARMAP=`LANG=${LANG} LC_ALL=${LC_ALL} LC_CTYPE=${LC_CTYPE} locale charmap`
			if [ "${CHARMAP}" = "UTF-8" ]
			then
				/usr/bin/unicode_start
			else
				/usr/bin/unicode_stop
			fi >/dev/null 2>&1
			true
		}

		# First mount /proc if necessary.
		unmount_proc="no"
		if [ ! -x /proc/1 ]
		then
			unmount_proc="yes"
			/bin/mount -n /proc
		fi

		if [ -f /proc/sys/dev/mac_hid/keyboard_sends_linux_keycodes ]
		then
			linux_keycodes=`/bin/cat /proc/sys/dev/mac_hid/keyboard_sends_linux_keycodes`
		else
			linux_keycodes=1
		fi

		# load new map
		if [ "${linux_keycodes}" -gt 0 ]
		then
			if [ -r "${CONFFILE}" ]
			then
				# Switch console mode to UTF-8 or ASCII as necessary
				unicode_start_stop
				if [ "${CONSOLE_TYPE}" = "serial" ]
				then
					/bin/loadkeys -q ${CONFFILE} 2>&1 >/dev/null
				else
					/bin/loadkeys -q ${CONFFILE}
				fi
				if [ ${?} -gt 0 ]
				then
					# if we've a serial console, we may not have a keyboard, so don't
					# complain if we fail.
					[ "${CONSOLE_TYPE}" = "serial" ] && exit 0
					echo "Problem when loading ${CONFDIR}/${CONFFILEROOT}.${EXT}.gz, use install-keymap"
					sleep 10
				fi
			fi
		fi

		# unmount /proc if we mounted it
		[ "${unmount_proc}" = "no" ] || /bin/umount -n /proc
	};
}
