virtual system/udev {
	critical;
	need = system/udev/filldev system/udev/udevd system/udev/set_hotplug;
}

# start the udev daemon
daemon system/udev/udevd {
	critical;
	need = system/udev/mountdev system/initial/mountvirtfs;	
	respawn;
	pid_of = udevd;
	forks;
	exec daemon = /sbin/udevd --daemon;
}

service system/udev/mountdev {
	critical;
	need = system/initial/mountvirtfs;
	script start = {
		error() {
			echo "${*}" >&2
			exit 1
		}

		[ -e /proc/filesystems ] || error "udev requires a mounted procfs, not started."
		/bin/grep -q '[[:space:]]tmpfs$' /proc/filesystems || error "udev requires tmpfs support, not started."
		[ -d /sys/block ] || error "udev requires a mounted sysfs, not started."

		# THIS service doesn't need this: (I'll remove it 12.1.) (deac)
		# [ -e /proc/sys/kernel/hotplug ] || error "udev requires hotplug support, not started."

		# mount a tmpfs over /dev, if somebody did not already do it
		/bin/grep -Eq "^[^[:space:]]+[[:space:]]+/dev[[:space:]]+tmpfs[[:space:]]+" /proc/mounts && exit 0

		# /dev/.static/dev is used by MAKEDEV to access the real /dev directory.
		# /etc/udev is recycled as a temporary mount point because it's the only
		# directory which is guaranteed to be available.
		/bin/mount -n --bind /dev /etc/udev
		if ! /bin/mount -n -o size=$tmpfs_size,mode=0755 -t tmpfs udev /dev
		then
			/bin/umount -n /etc/udev
			error "udev in /dev his own filesystem (tmpfs), not started."
		fi
		/bin/mkdir -p /dev/.static/dev
		/bin/chmod 700 /dev/.static/
		/bin/mount -n --move /etc/udev /dev/.static/dev

		# Make some default static onces, so we are sure they will exist.
		/bin/mknod /dev/null c 1 3
		/bin/mknod /dev/zero c 1 5
		/bin/mknod /dev/console c 5 1
		
		# Make sure all user may access /dev/null (Bug in Ubuntu)
		/bin/chmod 666 /dev/null

		# Send SIGHUP to initng pid 1, will reopen /dev/initctl and /dev/initng.
		/bin/kill -SIGHUP 1
		# Why? it's already mounted! (i'll remove it at 12.1., if no reason (deac))
		# [ -d /proc/1 ] || /bin/mount -n /proc
		exit 0
	};
}

# set the hotplug dispatcher
service system/udev/set_hotplug {
	critical;
	need = system/udev/udevd system/initial/mountvirtfs;
	script start = {
		# Make sure hotplug is not missing. The reason, because initng starts sulogin.
		[ -e /proc/sys/kernel/hotplug ] || echo "/proc/sys/kernel/hotplug is missing!"

		unamer=$(uname -r | /bin/sed -e '{ s/^[0-9][0-9]*\.[0-9][0-9]*\.//; s/[\.-].*// }')
		if [ "${unamer}" -ge 15 ]
		then
			# #425: Kernels >= 2.6.15 can directly read udev events via a netlink
			# socket. So there is no need anymore to run udevsend for every event.
			echo ""
		else
			# Setup hotplugging (if possible), if it can't it exits with error.
			echo "/sbin/udevsend"
		fi >/proc/sys/kernel/hotplug
	};
	script stop = {
		# every new hotplug-event can do something wrong.
		echo "" >/proc/sys/kernel/hotplug
	};
}

# fill /dev
service system/udev/filldev {
	critical;
	need = system/udev/udevd;
	script start = {
# debian
		if [ -e /etc/udev/links.conf ]
		then
			echo "Parsing /etc/udev/links.conf ..."
			/bin/grep '^[^#]' /etc/udev/links.conf | \
			while read type name arg1
			do
				[ "${type}" -a "${name}" -a ! -e "/dev/${name}" -a ! -L "/dev/${name}" ] || continue
				case "${type}" in
					L) /bin/ln -snf ${arg1} /dev/${name} & ;;
					D) /bin/mkdir -p /dev/${name} ;;
					M) /bin/mknod --mode=600 /dev/${name} ${arg1} &;;
					*) echo "links.conf: unparseable line (${type} ${name} ${arg1})" ;;
				esac
			done
		fi
		# A complete dir to copy
		if [ -d /lib/udev/devices ]
		then
			echo "Creating /dev from /lib/udev/devices ..."
			# Copy over default device tree
			{
				cd /lib/udev/devices &&
				#/usr/bin/find -print0 | /bin/cpio --quiet -0pmdu /dev
				tar c . | tar x --directory=/dev/
			} &
		fi
		if [ ! -e /etc/udev/links.conf -a ! -d /lib/udev/devices ]
		then
			# Some manually, youst to be sure.
			/bin/ln -snf /proc/self/fd /dev/fd &
			/bin/ln -snf fd/0 /dev/stdin &
			/bin/ln -snf fd/1 /dev/stdout &
			/bin/ln -snf fd/2 /dev/stderr &
			/bin/ln -snf /proc/kcore /dev/core &
			/bin/ln -snf /proc/asound/oss/sndstat /dev/sndstat &
		fi

		# which system provides udevsynthesize? gentoo does not.
		# and which provides udevplug? gentoo does not, too.

		# run syntesizers that will make hotplug events for every
		# devices that is currently in the computer, that will
		# create all dev files.
		if [ -x "/sbin/udevsynthesize" ]
		then
			udevd_timeout=300
			echo "Running /sbin/udevsynthesize to populate /dev ..."
			/bin/mkdir -p /dev/.udev/db /dev/.udev/queue /dev/.udevdb

			/sbin/udevsynthesize

			# wait for the udevd childs to finish
			echo "Waiting for /dev to be fully populated ..."
			while [ -d /dev/.udev/queue/ -a ${udevd_timeout} -ne 0 ]
			do
				sleep .2
				udevd_timeout=$((${udevd_timeout}-1))
			done
		elif [ -x "/sbin/udevplug" ]
		then
			/sbin/udevplug
		elif [ -x "/sbin/udevstart" ]
		then
			udevd_timeout=60
			/sbin/udevstart
			# wait for the udevd childs to finish
			echo "Waiting for /dev to be fully populated ..."
			while [ -d /dev/.udev/queue/ -a ${udevd_timeout} -ne 0 ]
			do
				sleep .2
				udevd_timeout=$((${udevd_timeout}-1))
			done
		fi
		chmod 0666 /dev/null
	};
}
